"use strict";

module.exports = {

  globals: {
    "Cc": false,
    "Ci": false,
    "Cu": false,
    "content": true,
    "dumpLog": false,
    "netscape": false,
    "addMessageListener": false,
    "goQuitApplication": false,
    "MozillaFileLogger": false,
    "Profiler": true,
    "Services": false,
    "gBrowser": false,
    "removeMessageListener": false,
    "sendAsyncMessage": false,
    "sendSyncMessage": false,
    "TalosPowersContent": true,
    "TalosPowersParent": true,
    "TalosContentProfiler": true,
    "TalosParentProfiler": true,
    "tpRecordTime": true
  },

  "plugins": [
    "mozilla"
  ],

  "rules": {
    "mozilla/avoid-Date-timing": "error"
  }
};
